@@ -101,8 +101,6 @@ build_opensbi() {
 		# Setup 64-bit platform list
 		BUILD_PLATFORM_SUBDIR+=("nuclei/ux600")
 		BUILD_PLATFORM_SUBDIR+=("kendryte/k210")
-		BUILD_PLATFORM_SUBDIR+=("fpga/ariane")
-		BUILD_PLATFORM_SUBDIR+=("fpga/openpiton")
 		BUILD_PLATFORM_SUBDIR+=("generic")
 		;;
 	*)
